* The RTS2 program is identical to the RTS1 program except it solves two areas at the same time.  Area 1 is the
* entire system such as WECC or ERCOT.  Area 2 is a sub area within area 1.  The RTS1 and RTS2 programs first
* calculate a COPT capacity outage probability table from Gdata input files.  Then every hour the LOLP in each
* area is looked up from the two tables F1 and F2.  If area 2 is more reliable than area 1 then area 2 is given
* an LOLP equal to area 1.  The transmission tie is assumed to not be binding for this situation.  Area 2 has
* transmission ties to area 1 that can limit the reliability of area 2.  The tie lines are modeled as a multistate
* generator.  The tie lines ATCs as entered as MW,Pr pairs.  The N-0 ATC with all lines in service is the highest
* MW value and all other ATCs with line outages are lower MWs.  We ignore ATCs with higher MW values than the N-0
* ATC.  Generators can be given up to nine extra states beyond the Up and Down states.  The hourly data holds the
* demand and variable energy resources as pu profiles and has a YYYYMMDDHH time stamp.  In the program below the
* area 1 and 2 variables are labeled so it is easy to identify which system the varible is to be used in.
* RTS2 was Written by Gene Preston http://egpreston.com and last updated on December 15, 2016 with changes.
* You may use this program as you wish provided you agree your findings are entirely your responsibilty.
*
      CHARACTER C221*220,D221*220,G1*32,H1*32,C2*2                      ! C is the current data and D is the previous C, G1 = generator data, H1 = hourly data
      CHARACTER C222*220,D222*220,G2*32,H2*32                           ! C is the current data and D is the previous C, G2 = generator data, H2 = hourly data
      REAL F1(0:190000)/190001*0./,SF1(20),VR1(20),PCTC1(20),VRES1(20)  ! F is the capacity outage probability table COPT  PCTC=%capacity credit  VRES=VR energy by source on NSH1 days
      REAL F2(0:190000)/190001*0./,SF2(20),VR2(20),PCTC2(20),VRES2(20)  ! F is the capacity outage probability table COPT  PCTC=%capacity credit  VRES=VR energy by source on NSH2 days
      REAL LFU1(7,2)/.006,.061,.242,.382,.242,.061,.006,7*0./           ! load forecast uncertainty probabilities and per unit load adjustments
      REAL LFU2(7,2)/.006,.061,.242,.382,.242,.061,.006,7*0./           ! load forecast uncertainty probabilities and per unit load adjustments
      REAL*8 LOLP1,LOLEAM1,LOLEPM1,LOLEV1,LOLH1,EUE1,ENGT1,VRE1,VREC1,  ! loss of load probability and expectation and loss of load hours are in double precision, total and variable resource energy
     & D1(19),LOLP24(24,2),PR1(12)                                      ! D1( ) dummy variables  LOLEAM1 is AM hours 1-12   LOLEPM1 is PM hours 13-24  LOLP24 used to write to file 7
      REAL*8 LOLP2,LOLEAM2,LOLEPM2,LOLEV2,LOLH2,LOLT2,ENGT2,VRE2,VREC2, ! loss of load probability and expectation and loss of load hours are in double precision, LOLT2 is loss of load hours due to ties
     & D2(19),DSF0/0.D0/,LOLE0/0.D0/,TLOLE2/0.D0/,DDSF/.02D0/,PR2(12)   ! D2( ) dummy variables  DSF0,LOLE0 = old values   LOLEAM2 is AM hours 1-12   LOLEPM2 is PM hours 13-24
      INTEGER IHA1/0/,IHB1/0/,JS/0/,MW(24,24,2),IP/0/,OUT1(12)          ! dummy hours used in LOLE AM PM tests, this line prevents a run time error  MW is for file 6 output  IP=0 do not print
      INTEGER IHA2/0/,IHB2/0/,OUT2(12)
      OPEN(5,FILE='OP.TXT')                                             ! open text output report file
      OPEN(6,FILE='OP.CSV')                                             ! open excel output report file
      OPEN(7,FILE='OPH.CSV')                                            ! we will write the houly LOLP's of significant days to this file
      WRITE(*,'(39HAREA 1 AND 2 GENERATOR DATA FILE NAMES:)')           ! area 1 is the larger area which contains all of area 2 generators
      READ(*,*) G1,G2                                                   ! read generator file names, must be no longer than 32 characters
      WRITE(*,'(31HAREA 2 TARGET LOLE (OR RETURN):)')
      H1=' '
      READ(*,'(A32)') H1
      IF(H1.NE.' ') READ(H1,*) TLOLE2                                   ! target LOLE desired, the DSF (demand scale factor) is changed to converge to a desired LOLE
      IF(TLOLE2.LE.0.) IP=1                                             ! there is no iterating to a solution so print all lines
      OPEN(3,FILE=G1,STATUS='OLD',ERR=16)                               ! open the area 1 generator data file
      OPEN(8,FILE=G2,STATUS='OLD',ERR=24)                               ! open the area 2 generator data file
      READ(3,'(A220)') C221                                             ! read the area 1 gen data title, note - any data beyond col 220 is skipped
      READ(8,'(A220)') C222                                             ! read the area 2 gen data title, note - any data beyond col 220 is skipped
      READ(3,*) H1                                                      ! read the area 1 HDATA file name
      READ(8,*) H2                                                      ! read the area 2 HDATA file name
      WRITE(5,'(/14H GDATA Files: ,2A32                                 ! print the generator input data file names
     &          /14H HDATA Files: ,2A32                                 ! print the hourly input data file names
     &          //1X,A220/1X,A220)') G1,G2,H1,H2,C221,C222              ! print the file names and the generation file titles
      WRITE(6,'(/14H GDATA Files: ,2A32                                 ! print the generator input data file names
     &          /14H HDATA Files: ,2A32                                 ! print the hourly input data file names
     &          //1X,A220/1X,A220)') G1,G2,H1,H2,C221,C222              ! print the file names and the generation file titles
      WRITE(7,'(/14H GDATA Files: ,2A32                                 ! print the generator input data file names
     &          /14H HDATA Files: ,2A32                                 ! print the hourly input data file names
     &          //1X,A220/1X,A220)') G1,G2,H1,H2,C221,C222              ! print the file names and the generation file titles
      OPEN(4,FILE=H1,STATUS='OLD',ERR=17)                               ! open the area 1 hourly load and variable resource file
      READ(4,'(A220)') C221                                             ! read the area 1 hourly data part of the study title
      C222=C221
      WRITE(5,'(1X,A220)') C221                                         ! print the area 1 hourly data file title
      WRITE(6,'(A220)') C221                                            ! print the area 1 hourly data file title
      WRITE(7,'(A220)') C221                                            ! print the area 1 hourly data file title
      READ(3,'(A220)',END=9) C221                                       ! read the area 1 demand multiplier and LFU
      READ(C221,*,ERR=9) DSF1,PLFU1,SLOLP1                              ! read the area 1 demand scale factor and the %LFU and the significant LOLP cutoff SLOLP
      READ(3,'(A220)') C221                                             ! read the area 1 number of VR
      READ(C221,*,END=9,ERR=9) NVS1                                     ! read the area 1 number of variable sources
      READ(3,'(A220)',END=9) C221                                       ! read the area 1 % capacity credit
      READ(C221,*,ERR=9) (PCTC1(I),I=1,NVS1)                            ! read the area 1 percent capacity credit for the reserve margin calculation
      READ(3,'(A220)',END=9) C221                                       ! read the area 1 scale factors for the VR
      READ(C221,*,ERR=9) (SF1(I),I=1,NVS1)                              ! read the area 1 scale factors for the variable resources
      DO 12 I=1,7                                                       ! area 1 LFU has 7 steps (load forecast uncertainty per Billington 1986)
  12  LFU1(I,2)=(I-4)*PLFU1/100.                                        ! create the area 1 delta loads for the 7 step table for a normal distribution
      IF(H2.NE.H1) THEN
        OPEN(9,FILE=H2,STATUS='OLD',ERR=25)                             ! open the area 2 hourly load and variable resource file
        READ(9,'(A220)') C222                                           ! read the area 2 hourly data part of the study title
        WRITE(5,'(1X,A220)') C222                                       ! print the area 2 hourly data file title
        WRITE(6,'(A220)') C222                                          ! print the area 2 hourly data file title
        WRITE(7,'(A220)') C222                                          ! print the area 2 hourly data file title
      ELSE
        WRITE(5,*) ' '
        WRITE(6,*) ' '
        WRITE(7,*) ' '
      ENDIF
      READ(8,'(A220)',END=9) C222                                       ! read the area 2 demand multiplier and LFU
      READ(C222,*,ERR=26) DSF2,PLFU2,SLOLP2                             ! read the area 2 demand scale factor and the %LFU and the significant LOLP cutoff SLOLP
      READ(8,'(A220)') C222                                             ! read the area 2 number of VR
      READ(C222,*,END=26,ERR=26) NVS2                                   ! read the area 2 number of variable sources
      READ(8,'(A220)',END=26) C222                                      ! read the area 2 % capacity credit
      READ(C222,*,ERR=26) (PCTC2(I),I=1,NVS2)                           ! read the area 2 percent capacity credit for the reserve margin calculation
      READ(8,'(A220)',END=26) C222                                      ! read the area 2 scale factors for the VR
      READ(C222,*,ERR=26) (SF2(I),I=1,NVS2)                             ! read the area 2 scale factors for the variable resources
      DO 30 I=1,7                                                       ! area 1 LFU has 7 steps (load forecast uncertainty per Billington 1986)
  30  LFU2(I,2)=(I-4)*PLFU2/100.                                        ! create the area 1 delta loads for the 7 step table for a normal distribution
      READ(3,'(A220)') C221                                             ! skip this area 1 header for generators
      READ(8,'(A220)') C222                                             ! skip this area 2 header for generators
* calculate the COPT, the capacity outage probability table F1
      MAXGEN1=0                                                         ! MAXGEN is the sum of CAPA = total installed generation MW
      MXF1=0                                                            ! we store the maximum size of F that is needed for F1(x)>1.E-16
      RMGEN1=0.                                                         ! reserve margin generation used in the percent reserve calculation
      DO 1 I=1,99999
      READ(3,'(A220)',END=10) C221                                      ! read the generator data to the end of the file
      READ(C221,*,ERR=9,END=9) J,C2,C2,CAPA,RM,FOR,K,                   ! read bus#, genname, gentype, gen max MW, reserve margin %, forced outage rate, # extra states and then those states
     & (OUT1(L),PR1(L),L=3,K+2)
      PR1(1)=1.-FOR                                                     ! this is the up state which is not shifted and it has to be first in the PR array
      OUT1(1)=0                                                         ! the UP state at CAPA MW level has 0 MW outaged
      PR1(2)=FOR                                                        ! this is the total outage state where there is no power from the generator at all
      OUT1(2)=CAPA                                                      ! the entire generator CAPA capacity is outaged with Pr=FOR
      IF(K.GT.0) THEN                                                   ! set up the rest of the OUT,PR variables
        DO 6 L=3,K+2
        PR1(1)=PR1(1)-PR1(L)                                            ! additional states subtract from the probability of the UP state
        IF(PR1(1).LT.0.) GOTO 9                                         ! cannot have a negative UP probability
        OUT1(L)=CAPA-OUT1(L)                                            ! convert the availability MW to an outaged MW, this is what is convolved
        IF(OUT1(L).LE.0) GOTO 9
   6    CONTINUE
      ENDIF
      RMGEN1=RMGEN1+CAPA*RM/100.                                        ! rerate generator's capacities for the reserve margin calculation
      MAXGEN1=MAXGEN1+CAPA                                              ! MAXGEN is the total installed generation MW
      MG1=MAXGEN1                                                       ! we can start sweeping across F where F is going to be >0.
      IF(MG1.GT.190001) MG1=190001                                      ! its ok for the MAXGEN to exceed 190001
      DO 1 IX=MG1,0,-1                                                  ! sweep from right to left across F to convolve the next generator
      F1(IX)=F1(IX)*PR1(1)                                              ! scale down the UP state which is not shifted
      DO 23 L=2,K+2                                                     ! each of the rest of the states will be an outage state
      J=IX-OUT1(L)                                                      ! J will point to the shifted to the right generator outage state
      FJ=PR1(L)                                                         ! all F1(ix<0) = 1
      IF(J.GE.0) FJ=F1(J)*FJ                                            ! ix is => 0
      IF(FJ.LT.1.E-36) FJ=0.                                            ! this prevents an underflow error message
  23  F1(IX)=F1(IX) + FJ                                                ! down states are shifted to the right and added to the scaled down up state which was not moved
      IF(F1(IX).GE.1.E-16.AND.MXF1.LT.IX) MXF1=IX                       ! capture the largest IX needed, ignore smaller F
   1  CONTINUE
  10  CLOSE(3)
* calculate the COPT, the capacity outage probability table F2
      MAXGEN2=0                                                         ! MAXGEN is the sum of CAPA = total installed generation MW
      MXF2=0                                                            ! we store the maximum size of F that is needed for F1(x)>1.E-16
      RMGEN2=0.                                                         ! reserve margin generation used in the percent reserve calculation
      DO 31 I=1,99999
      READ(8,'(A220)',END=34) C222                                      ! read the generator data to the end of the file
      READ(C222,*,ERR=26,END=26) J,C2,C2,CAPA,RM,FOR,K,                 ! read bus#, genname, gentype, gen max MW, reserve margin %, forced outage rate, # extra states and then those states
     & (OUT2(L),PR2(L),L=3,K+2)
      PR2(1)=1.-FOR                                                     ! this is the up state which is not shifted and it has to be first in the PR array
      OUT2(1)=0                                                         ! the UP state at CAPA MW level has 0 MW outaged
      PR2(2)=FOR                                                        ! this is the total outage state where there is no power from the generator at all
      OUT2(2)=CAPA                                                      ! the entire generator CAPA capacity is outaged with Pr=FOR
      IF(K.GT.0) THEN                                                   ! set up the rest of the OUT,PR variables
        DO 32 L=3,K+2
        PR2(1)=PR2(1)-PR2(L)                                            ! additional states subtract from the probability of the UP state
        IF(PR2(1).LT.0.) GOTO 26                                        ! cannot have a negative UP probability
        OUT2(L)=CAPA-OUT2(L)                                            ! convert the availability MW to an outaged MW, this is what is convolved
        IF(OUT2(L).LE.0) GOTO 26
  32    CONTINUE
      ENDIF
      RMGEN2=RMGEN2+CAPA*RM/100.                                        ! rerate generator's capacities for the reserve margin calculation
      MAXGEN2=MAXGEN2+CAPA                                              ! MAXGEN is the total installed generation MW
      MG2=MAXGEN2                                                       ! we can start sweeping across F where F is going to be >0.
      IF(MG2.GT.190001) MG2=190001                                      ! its ok for the MAXGEN to exceed 190001
      DO 31 IX=MG2,0,-1                                                 ! sweep from right to left across F to convolve the next generator
      F2(IX)=F2(IX)*PR2(1)                                              ! scale down the UP state which is not shifted
      DO 33 L=2,K+2                                                     ! each of the rest of the states will be an outage state
      J=IX-OUT2(L)                                                      ! J will point to the shifted to the right generator outage state
      FJ=PR2(L)                                                         ! all F1(ix<0) = 1
      IF(J.GE.0) FJ=F2(J)*FJ                                            ! ix is => 0
      IF(FJ.LT.1.E-36) FJ=0.                                            ! this prevents an underflow error message
  33  F2(IX)=F2(IX) + FJ                                                ! down states are shifted to the right and added to the scaled down up state which was not moved
      IF(F2(IX).GE.1.E-16.AND.MXF2.LT.IX) MXF2=IX                       ! capture the largest IX needed, ignore smaller F
  31  CONTINUE
  34  WRITE(5,'(/12H MAX GENR  =,I11, 3H MW,
     &       19X,12H MAX GENR  =,I11, 3H MW)') MAXGEN1,MAXGEN2          ! MAXGEN is the installed generation capacity
      CLOSE(8)
* read the hourly data files
      DO 15 ITER=1,20                                                   ! if it can't find the correct demand in 20 attempts then stop
      IF(ITER.EQ.20) IP=1                                               ! print out something even if not converged
      REWIND(4)
      IF(H2.NE.H1) REWIND(9)
      DO 14 I=1,14
      D1(I)=0.
  14  D2(I)=0.                                                          ! zero out all the dummy variables starting a new study
      READ(4,'(A220)') C221                                             ! read HDATA title again
      READ(4,'(A220)',END=9) C221
      C222=C221
      IF(C221(1:10).NE.'YYYYMMDDHH')                                    ! read as an integer I the data is good through 2147123124 but fails on year 2148.
     &  WRITE(5,'(23HYYYYMMDDHH out of place)')                         ! just a check to make sure were are reading the data in the correct order
      IF(ITER.EQ.1) THEN
        WRITE(7,'(/11X,14HYear,MoDaHr D,8X,14HLOLP,  NETDEM,A206)')     ! new YYYYMMDDHH line header for OPH report
     &    C222(14:220)
      ENDIF
      IF(H2.NE.H1) THEN
        READ(9,'(A220)') C222                                           ! read HDATA title again
        READ(9,'(A220)',END=26) C222
        IF(C222(1:10).NE.'YYYYMMDDHH')
     &  WRITE(5,'(23HYYYYMMDDHH out of place)')                         ! just a check to make sure were are reading the data in the correct order
      ENDIF
      VRC1=0.                                                           ! VRC will save the variable resource capacity MW credit
      VRC2=0.
      DO 11 I=1,NVS1
      VRES1(I)=0.                                                       ! zero out the variable resource energy sum by source on significant days NSH1
      VRES2(I)=0.
      VRC1=VRC1+SF1(I)*PCTC1(I)/100.                                    ! MW variable resource capacity, we don't add uncertainty here since the actual data already contains the uncertainty
  11  VRC2=VRC2+SF2(I)*PCTC2(I)/100.
      IF(ITER.EQ.1) THEN
       WRITE(5,'(12H VARBL RES =,I11, 3H MW,
     &       19X,12H VARBL RES =,I11, 3H MW)') INT(VRC1+.5),INT(VRC2+.5)! variable resource total MW capacity credit
       WRITE(5,'(12H GENR+VRES =,I11, 3H MW,
     &       19X,12H GENR+VRES =,I11, 3H MW)') MAXGEN1+INT(VRC1+.5),    ! the sum of the GENR and VR capacities
     &       MAXGEN2+INT(VRC2+.5)
       WRITE(5,'(12H MAX COPT  =,I11,13H (190000 MAX),
     &        9X,12H MAX COPT  =,I11,13H (190000 MAX))') MXF1,MXF2      ! shows how much of F1(x) and F2(x) array space was needed in this problem, COPT = capacity outage probability table
      ENDIF
      READ(4,'(A220)',END=9)  C221                                      ! read the first line of hourly data
      C222=C221
      IF(H2.NE.H1) READ(9,'(A220)',END=26) C222                         ! read the first line of hourly data
      D221=C221                                                         ! copy the first line of data to D221
      D222=C222
      READ(C221,'(I4)') IFY                                             ! first year in HDATA1
      READ(C222,'(I4)') IY                                              ! first year in HDATA2
      IF(IFY.NE.IY) GOTO 26                                             ! the Hdata file years must match
* repeat years as many times as there is annual data
      IYR=IFY-1                                                         ! we only have to track one file's years since both have to match at every hour
      NTH=0                                                             ! number of total hours - should be the same in both hourly files
      DO 7 IY=1,9999                                                    ! simulate up to 9999 years of historical data
      IYR=IYR+1
      NSH1=0
      NSH2=0                                                            ! number of significant hours in area 2, where LOLP2>SLOLP2 significant LOLP
      LOLEAM1=0.D0              ! these indices will be created         ! this is the annual sum of LOLP peaks occurring in the AM hours 1-12
      LOLEAM2=0.D0
      LOLEPM1=0.D0              ! and summed in 7 LFU steps             ! this is the annual sum of LOLP peaks occurring in the PM hours 13-24
      LOLEPM2=0.D0
      LOLEV1=0.D0                                                       ! loss of load events per year
      LOLEV2=0.D0
      LOLH1=0.D0                                                        ! loss of load hours is the sum of every hour's LOLP for a year
      LOLH2=0.D0
      LOLT2=0.D0                                                        ! loss of load hours due to transmission ties being more limiting than area 1 supply
      EUE1 =0.D0                                                        ! EUE (or EENS) is the expected unserved energy of expected energy not served
      ENGT1=0.D0                                                        ! this is the total energy served used in calculating the % unserved energy
      ENGT2=0.D0
      VRE1 =0.D0                                                        ! annual variable resource energy
      VRE2 =0.D0
      VREC1=0.D0                                                        ! annual variable resource energy load, dumped, or curtailed
      VREC2=0.D0
* sweep through all the hours calculating the LOLP each hour and the LOLE and LOLH
      IHRS=0                                                            ! count the hours per day and reset to define a new day, capture the max LOLP
      IHB1OLD=0
      IHB2OLD=0
      PLOLPAM1=0              ! the lolps will be calculated for        ! PLOLPAM1 will be the peak AM period LOLP of each day if the AM has a peak
      PLOLPAM2=0
      PLOLPPM1=0              ! each of the 7 steps and summed          ! PLOLPPM1 will be the peak PM period LOLP of each day if the PM has a peak
      PLOLPPM2=0
      PKDEM1=0.               ! so only the lolp is affected by         ! capture the peak demand before renewables are applied
      PKDEM2=0.
      PKNET1=0.               ! the LFU                                 ! capture the peak net demand after renewables are applied
      PKNET2=0.
      IH3=0                   ! the hour during the day, we do this separately here because the YYYYMMDDHH can contain leap days with a screwed up hour
      IOK=0                   ! IOK=1 means list this day in file 6
      DO 4 IH=1,9999                                                    ! assume more hours than actual and note change in YY as start of a new year
      NTH=NTH+1                                                         ! total number of hours
      NH=IH
      READ(C221(14:220),*,ERR=9)  DEMD1,(VR1(I),I=1,NVS1)               ! read the hourly demand and variable sources, YYYYMMDDHH and D are just for our reference
      READ(C222(14:220),*,ERR=26) DEMD2,(VR2(I),I=1,NVS2)               ! read the hourly demand and variable sources, YYYYMMDDHH and D are just for our reference
      HD1=DEMD1*DSF1                                                    ! hourly MW demand is the first number read, the rest are variable sources
      HD2=DEMD2*DSF2
      IH3=IH3+1
      READ(C221,*) MW(23,IH3,1),MW(24,IH3,1)                            ! capture the YYYYMMDDHH in 23, D in 24, 22 net demand, 21 demand, VR in 1-20
      READ(C222,*) MW(23,IH3,2),MW(24,IH3,2)                            ! capture the YYYYMMDDHH in 23, D in 24, 22 net demand, 21 demand, VR in 1-20
      MW(21,IH3,1)=HD1+.5                                               ! capture the MW demand for hour IH3
      MW(21,IH3,2)=HD2+.5
      IF(HD1.GT.PKDEM1) PKDEM1=HD1                                      ! capture the peak demand for this year before renewables are applied
      IF(HD2.GT.PKDEM2) PKDEM2=HD2
      HD11=HD1                 ! the 7 step LFU will be applied to HD11 ! remember the native demand for this hour
      HD12=HD2
      ENGT1=ENGT1+HD1          ! only in the LOLP lookup in F1(x)       ! this is the total energy of the original hourly demand
      ENGT2=ENGT2+HD2          ! only in the LOLP lookup in F2(x)       ! this is the total energy of the original hourly demand
      IF(NVS1.GT.0) THEN       ! and not applied to other values        ! see if any variable sources are present
        DO 2 I=1,NVS1          ! everywhere the LOLP is used, the       ! if there are varible sources then loop through all of them
        HD1=HD1-VR1(I)*SF1(I)  ! the LFU will be carried forward        ! subtract the scaled variable source from the scaled hourly demand
   2    MW(I,IH3,1)=VR1(I)*SF1(I)+.5                                    ! capture the MW values of the VR this hour
      ENDIF
      IF(NVS2.GT.0) THEN       ! and not applied to other values        ! see if any variable sources are present
        DO 28 I=1,NVS2         ! everywhere the LOLP is used, the       ! if there are varible sources then loop through all of them
        HD2=HD2-VR2(I)*SF2(I)  ! the LFU will be carried forward        ! subtract the scaled variable source from the scaled hourly demand
   28   MW(I,IH3,2)=VR2(I)*SF2(I)+.5                                    ! capture the MW values of the VR this hour
      ENDIF
      IF(HD1.GT.0.) THEN
        MW(22,IH3,1)=HD1+.5
      ELSE
        MW(22,IH3,1)=HD1-.5
      ENDIF
      IF(HD2.GT.0.) THEN
        MW(22,IH3,2)=HD2+.5
      ELSE
        MW(22,IH3,2)=HD2-.5
      ENDIF
      IF(HD1.LT.0.) THEN
        VREC1=VREC1-HD1                                                 ! remember the total renewable energy that has been dumped
        HD1=0.                                                          ! however, if the net hourly demand is negative, then set the demand to 0 MW
      ENDIF
      IF(HD2.LT.0.) THEN
        VREC2=VREC2-HD2                                                 ! remember the total renewable energy that has been dumped
        HD2=0.                                                          ! however, if the net hourly demand is negative, then set the demand to 0 MW
      ENDIF
      IF(HD1.GT.PKNET1) PKNET1=HD1                                      ! capture the net peak demand for this year after renewables are applied
      IF(HD2.GT.PKNET2) PKNET2=HD2
      VRE1=VRE1+(HD11-HD1)                                              ! sum up the total variable source energy that has been applied to the demand
      VRE2=VRE2+(HD12-HD2)
      LOLP1=0.D0                                                        ! getting ready to sum the LOLPs for the 7 LFU steps
      LOLP2=0.D0
      DO 13 L=1,7                                                       ! now we loop through all the LFU steps
      HD21=HD1+HD11*LFU1(L,2)                                           ! HD12 is a new modified demand that contains the load undertainty
      HD22=HD2+HD12*LFU2(L,2)
      IF(HD21.LT.0.) HD21=0.                                            ! let's just avoid feeding negative load to the COPT, the LOLP is zero anyway
      IF(HD22.LT.0.) HD22=0.
      IHD1=HD21                                                         ! this is the integer part of HD12 to the left of the decimal
      IHD2=HD22
      P=HD21-IHD1                                                       ! P is the fraction of MW < 1 MW greater than the integer HD1 MW
      FI=0.                                                             ! assume area 1 has LOLP=0 for this LFU
      FJ=0.                                                             ! assume area 2 has LOLP=0 for this LFU
      IF(MAXGEN1-IHD1.LE.MXF1) THEN                                     ! assume F1(X>MXF)=0  MAXGEN minus integer part of the demand on the COPT
        D1(15)=0.D0                                                     ! sum up the unserved energy for this hour, D1(15) is just a dummy variable
        DO 3 I=MXF1,MAXGEN1-IHD1,-1                                     ! this loop is to sum up the unserved energy under the COPT
        FI=1.                                                           ! assume area 1 has LOLP=1 for this LFU (note this is just for EUE)
        IF(I.GE.0) FI=F1(I)                                             ! the actual LOLP falls between 0 and 1
   3    D1(15)=D1(15)+FI                                                ! 1 MW wide increments of load with F probability not served starting with the smallest of F
        IF(P.GT.0.) THEN                                                ! this is the small fraction of load < 1 MW but greater than the IHD1 integer MW load
          IHD1=IHD1+1                                                   ! pick up the LOLP for the next higher MW load level, its a stairstep
          FI=1.                                                         ! this use of FI is just for the EUE
          IF(MAXGEN1-IHD1.GE.0) FI=F1(MAXGEN1-IHD1)                     ! since P>0 we have to step one MW higher on the F1 load level
          D1(15)=D1(15)+P*FI                                            ! and add on that bit of unserved energy
        ENDIF
        EUE1=EUE1+D1(15)*LFU1(L,1)                                      ! add on this hour's unserved energy D1(15) to the running sum EUE (this doesn't work for area 2)
        FI=1.                                                           ! assume LOLP=1 as we recalculate the FI
        IF(MAXGEN1-IHD1.GE.0) FI=F1(MAXGEN1-IHD1)                       ! but we find area 1 LOLP is between 0 and 1 for this particular LFU for this hour
        LOLP1=LOLP1+FI*LFU1(L,1)                                        ! the LOLP is a lookup for the higher demand if P>0, this line works also for P=0
      ENDIF
      P=HD22-IHD2                                                       ! P is the fraction of MW < 1 MW greater than the integer HD1 MW
      IF(MAXGEN2-IHD2.LE.MXF2) THEN                                     ! assume F1(X>MXF)=0  MAXGEN minus integer part of the demand on the COPT
        IF(P.GT.0.) IHD2=IHD2+1                                         ! pick up the LOLP for the next higher MW load level, its a stairstep
        FJ=1.                                                           ! assume area 2 LOLP=1 for this area 2 LFU
        IF(MAXGEN2-IHD2.GE.0) FJ=F2(MAXGEN2-IHD2)                       ! area 2 has an LOLP between 0 and 1 for this LFU
        IF(FI.LT.FJ) THEN                                               ! see if area 1 LOLP < area 2 LOLP and if so the transmission ties are being used
          LOLT2=LOLT2+(FJ-FI)*LFU2(L,1)                                 ! add up the hours of needing the tie lines, this is calculated in the spirit of the LOLH
          LOLP2=LOLP2+FJ*LFU2(L,1)                                      ! the area 2 LOLP is > area 1 LOLP so the tie lines are being used for sure
        ELSE
          LOLP2=LOLP2+FI*LFU2(L,1)                                      ! if area 1 LOLP > area 2 LOLP the external supply is limiting and not the tie lines
        ENDIF
      ENDIF
  13  CONTINUE                                                          ! end of the LFU and LOLP calculations
      IF(LOLP1.GE.SLOLP1) THEN
        NSH1=NSH1+1                                                     ! a significant hour LOLP=>significant LOLP
*        IOK=1                                                           ! remember this day is significant for printing to OPH.CSV  - NSH1 is not used in RTS2
        DO 22 I=1,NVS1
  22    VRES1(I)=VRES1(I)+VR2(I)*SF1(I)
      ENDIF
      IF(LOLP2.GE.SLOLP2) THEN
        NSH2=NSH2+1                                                     ! a significant hour LOLP=>significant LOLP
        IOK=1                                                           ! remember this day is significant for printing to OPH.CSV
        DO 27 I=1,NVS2
  27    VRES2(I)=VRES2(I)+VR2(I)*SF2(I)                                 ! sum the MW of VR for significant hours so we can average the MW of each VR during these hours
      ENDIF
      LOLP24(IH3,1)=LOLP1
      LOLP24(IH3,2)=LOLP2
      IF(IOK.EQ.1.AND.(IH3.EQ.24.OR.C222(9:10).EQ.'24')) THEN
        DO 21 I=1,24
        WRITE(G2,'(I10)') MW(23,I,2)
        IF(IP.EQ.1) WRITE(7,'(A4,2H, A6,I4,1H,F8.6,1H,21(I9,1H,))')
     &    G2(1:4),G2(5:10),MW(24,I,2),LOLP24(I,2),MW(22,I,2),MW(21,I,2),
     &    (MW(J,I,2),J=1,NVS2)
  21    CONTINUE
      ENDIF
      IF(IH3.EQ.24.OR.C222(9:10).EQ.'24') THEN
        IOK=0
        IH3=0
      ENDIF
      LOLH1=LOLH1+LOLP1                                                 ! calculate the annual loss of load hours
      LOLH2=LOLH2+LOLP2
      IHRS=IHRS+1                                                       ! this is the hour of the day
      IF(IHRS.LE.12) THEN                                               ! the AM period
        IF(LOLP1.GT.PLOLPAM1) THEN
          PLOLPAM1=LOLP1                                                ! capture the max LOLP1 each day
          IHA1=IHRS                                                     ! remember the hour of the peak AM LOLP
        ENDIF
        IF(LOLP2.GT.PLOLPAM2) THEN
          PLOLPAM2=LOLP2                                                ! capture the max LOLP1 each day
          IHA2=IHRS                                                     ! remember the hour of the peak AM LOLP
        ENDIF
      ELSE                                                              ! the else will be hours 13-24
        IF(LOLP1.GT.PLOLPPM1) THEN
          PLOLPPM1=LOLP1                                                ! capture the max LOLP2 each day
          IHB1=IHRS                                                     ! remember the hour of the peak PM LOLP
        ENDIF
        IF(LOLP2.GT.PLOLPPM2) THEN
          PLOLPPM2=LOLP2                                                ! capture the max LOLP2 each day
          IHB2=IHRS                                                     ! remember the hour of the peak PM LOLP
        ENDIF
      ENDIF
      IF(IHRS.EQ.24) THEN                                               ! we are at the end of a day
        IF(IHB1-IHA1.LE.6.OR.24-IHB1OLD+IHA1.LE.6) THEN                 ! peak LOLPs are too close together to treat as separate events
          IF(PLOLPPM1.GE.PLOLPAM1) PLOLPAM1=0.
          IF(PLOLPAM1.GE.PLOLPPM1) THEN
            IHB1=15                                                     ! make sure its not close to midnight is all
            PLOLPPM1=0.
          ENDIF
        ENDIF
        IF(IHB2-IHA2.LE.6.OR.24-IHB2OLD+IHA2.LE.6) THEN                 ! peak LOLPs are too close together to treat as separate events
          IF(PLOLPPM2.GE.PLOLPAM2) PLOLPAM2=0.
          IF(PLOLPAM2.GE.PLOLPPM2) THEN
            IHB2=15                                                     ! make sure its not close to midnight is all
            PLOLPPM2=0.
          ENDIF
        ENDIF
        LOLEV1=LOLEV1+PLOLPAM1+PLOLPPM1                                 ! loss of load events can count up to two events per day if they meet the above tests
        LOLEV2=LOLEV2+PLOLPAM2+PLOLPPM2
        IF(PLOLPPM1.GE.PLOLPAM1) PLOLPAM1=0.                            ! this goes ahead and resets the AM and PM max LOLPs so there is only one per day
        IF(PLOLPPM2.GE.PLOLPAM2) PLOLPAM2=0.
        IF(PLOLPAM1.GE.PLOLPPM1) PLOLPPM1=0.
        IF(PLOLPAM2.GE.PLOLPPM2) PLOLPPM2=0.
        LOLEAM1=LOLEAM1+PLOLPAM1                                        ! the LOLEAM1 is the sum of daily maximum AM LOLPs
        LOLEAM2=LOLEAM2+PLOLPAM2
        LOLEPM1=LOLEPM1+PLOLPPM1                                        ! the LOLEPM1 is the sum of daily maximum PM LOLPs
        LOLEPM2=LOLEPM2+PLOLPPM2
        PLOLPAM1=0.                                                     ! reset PLOLPAM1 for a new day
        PLOLPAM2=0.
        PLOLPPM1=0.                                                     ! reset PLOLPPM1 for a new day
        PLOLPPM2=0.
        IHRS=0                                                          ! reset the hours counter for a new day
        IHB1OLD=IHB1                                                    ! this it the hour of max LOLP in the PM of the previous day
      ENDIF
      C221=' '
      READ(4,'(A220)',END=5) C221                                       ! read a new line of data or exit to the final report
      C222=C221
      IF(H2.NE.H1) READ(9,'(A220)',END=5) C222                          ! read a new line of data or exit to the final report
      IF(C221(1:12).NE.C222(1:12)) GOTO 26                              ! the date time stamp must agree exactly every hour
      IF(C221(1:4).NE.D221(1:4)) GOTO 5                                 ! we must have started a new year if the YY is different, go to the report
      D221=C221                                                         ! copy the line of data to D221 for this hour
      D222=C222
   4  CONTINUE                                                          ! end of the hour loop, do the next hour, which we never reach the end of this loop
   5  NY=IY                                                             ! remember the number of years
      IF(IP.EQ.1) THEN
        WRITE(5,'(/I5,9H RESULTS:,31X,I5,9H RESULTS:)') IYR,IYR
        WRITE(5,'(12H PERIOD    =,I11,6H HOURS,
     &        16X,12H PERIOD    =,I11,6H HOURS)') NH,NH
        WRITE(5,'(12H PEAK DEMD =,F11.3,3H MW,
     &        19X,12H PEAK DEMD =,F11.3,3H MW)') PKDEM1,PKDEM2
        RM1=(RMGEN1+VRC1)/PKDEM1*100.-100.                              ! reserve margin calculation of the total system
        RM2=(RMGEN2+VRC2)/PKDEM2*100.-100.                              ! reserve margin calculation of area 2 plus ties
        WRITE(5,'(12H RESERVE   =,F11.1,2H %,
     &        20X,12H RESERVE   =,F11.1,2H %)') RM1,RM2
        WRITE(5,'(12H LOAD ENGY =,F11.0,4H GWh,
     &        18X,12H LOAD ENGY =,F11.0,4H GWh)')ENGT1/1000.,ENGT2/1000.
        WRITE(5,'(12H LOAD FACT =,F11.3,2H %,
     &        20X,12H LOAD FACT =,F11.3,2H %)') 100.*ENGT1/(NH*PKDEM1),
     &        100.*ENGT2/(NH*PKDEM2)
        WRITE(5,'(12H PEAK NETD =,F11.3,13H MW (DEMD-VR),
     &         9X,12H PEAK NETD =,F11.3,13H MW (DEMD-VR))')PKNET1,PKNET2
        WRITE(5,'(12H VR E USED =,F11.0,4H GWh,
     &        18X,12H VR E USED =,F11.0,4H GWh)') VRE1/1000.,VRE2/1000.
        WRITE(5,'(12H VR E LOST =,F11.0,4H GWh,
     &        18X,12H VR E LOST =,F11.0,4H GWh)')VREC1/1000.,VREC2/1000.
        WRITE(5,'(14H Indices for a,F6.2,6H% LFU:,
     &        19X,14H Indices for a,F6.2,6H% LFU:)') PLFU1,PLFU2
        WRITE(5,'(12H LOLE(AM)  =,F11.6,4H d/y,
     &        18X,12H LOLE(AM)  =,F11.6,4H d/y)') LOLEAM1,LOLEAM2       ! the AM LOLE
        WRITE(5,'(12H LOLE(PM)  =,F11.6,4H d/y,
     &        18X,12H LOLE(PM)  =,F11.6,4H d/y)') LOLEPM1,LOLEPM2       ! the PM LOLE
        WRITE(5,'(12H LOLE      =,F11.6,4H d/y,
     &        18X,12H LOLE      =,F11.6,4H d/y)') LOLEAM1+LOLEPM1,      ! LOLE = 1.368863 which agrees exactly with the 1986 paper.
     &        LOLEAM2+LOLEPM2
        WRITE(5,'(12H LOLEV **  =,F11.6,9H events/y,
     &        13X,12H LOLEV **  =,F11.6,9H events/y)') LOLEV1,LOLEV2    ! loss of load events
        WRITE(5,'(12H LOLEMW    =,F11.0,3H MW,                          ! the expected MW loss of load for area 1
     &        19X,12H TIE LOLE  =,F11.6,4H d/y)') EUE1/LOLH1,
     &        LOLEAM2+LOLEPM2-LOLEAM1-LOLEPM1                           ! the increase in area 2 LOLE due to the ties
        WRITE(5,'(12H LOLH      =,F11.6,4H h/y,
     &        18X,12H LOLH      =,F11.6,4H h/y)') LOLH1,LOLH2           ! LOLH =   9.394175 which agrees exactly with the 1986 paper.
        WRITE(5,'(12H LOLH/event=,F11.6,8H h/event,
     &        14X,12H LOLH/event=,F11.6,8H h/event)') LOLH1/LOLEV1,     ! duration of loss of load in hours per event
     &        LOLH2/LOLEV2
        WRITE(5,'(12H EUE MWh   =,F11.0,4H MWh,
     &        18X,12H TIE LINES =,F11.3,4H h/y)') EUE1,LOLT2
        WRITE(5,'(12H EUE ppm   =,F11.2,7H pu ppm,
     &        15X,12H TIE LOLE  =,F11.1,2H %)') EUE1/ENGT1*1.E6,
     &        (LOLEAM2+LOLEPM2)/(LOLEAM1+LOLEPM1)*100.-100.             ! the % increase in area 2 LOLE due to the ties
        WRITE(5,'(12H EUE %     =,F11.6,2H %,
     &        20X,12H TIE LOLH  =,F11.1,2H %)') EUE1/ENGT1*100.,        ! % unserved energy, .007690 is consistent with EUE of 1.176 in the 1986 paper
     &        LOLH2/LOLH1*100.-100.                                     ! the % increase in area 2 LOLH due to the ties
        WRITE(5,'(6H LOLP>,1PE5.0E1,1H=,I11  ,4H hrs,
     &        18X,6H LOLP>,1PE5.0E1,1H=,I11  ,4H hrs)') SLOLP1,NSH1,    ! significant LOLP and number of hours with significant LOLPs
     &        SLOLP2,NSH2
      ENDIF
      D1(1)=D1(1)+PKDEM1
      D1(2)=D1(2)+ENGT1/1000.
      D1(3)=D1(3)+100.*ENGT1/(NH*PKDEM1)
      D1(4)=D1(4)+PKNET1
      D1(5)=D1(5)+VRE1/1000.
      D1(6)=D1(6)+VREC1/1000.
      D1(7)=D1(7)+LOLEAM1
      D1(8)=D1(8)+LOLEPM1
      D1(9)=D1(9)+LOLEV1
      D1(10)=D1(10)+LOLH1
      D1(11)=D1(11)+EUE1
      D1(12)=D1(12)+ENGT1
      D1(13)=D1(13)+NSH1                                                ! total number of significant hours for the entire period
      D1(14)=D1(14)+RM1                                                 ! reserve margin calculation
      D2(1)=D2(1)+PKDEM2
      D2(2)=D2(2)+ENGT2/1000.
      D2(3)=D2(3)+100.*ENGT2/(NH*PKDEM2)
      D2(4)=D2(4)+PKNET2
      D2(5)=D2(5)+VRE2/1000.
      D2(6)=D2(6)+VREC2/1000.
      D2(7)=D2(7)+LOLEAM2
      D2(8)=D2(8)+LOLEPM2
      D2(9)=D2(9)+LOLEV2
      D2(10)=D2(10)+LOLH2
      D2(11)=D2(11)+LOLT2
      D2(12)=D2(12)+ENGT2
      D2(13)=D2(13)+NSH2                                                ! total number of significant hours for the entire period
      D2(14)=D2(14)+RM2                                                 ! reserve margin calculation
      IF(C221(1:3).EQ.'end') GOTO 8                                     ! we reached the end of the datain file
      D221=C221                                                         ! remember C221 for the new year set of data
      D222=C222
   7  CONTINUE
   8  LOLEAM1=(D1(7)+D1(8))/NY
      LOLEAM2=(D2(7)+D2(8))/NY
      IF(NY.GT.1.AND.IP.EQ.1) THEN
        WRITE(5,'(/16H AVERAGE VALUES:
     &         29X,16H AVERAGE VALUES:)')
        WRITE(5,'(12H PERIOD    =,I11,6H HOURS,
     &        16X,12H PERIOD    =,I11,6H HOURS)') NTH/NY,NTH/NY
        WRITE(5,'(12H PEAK DEMD =,F11.3,3H MW,
     &        19X,12H PEAK DEMD =,F11.3,3H MW)') D1(1)/NY,D2(1)/NY
        WRITE(5,'(12H RESERVE   =,F11.1,2H %,
     &        20X,12H RESERVE   =,F11.1,2H %)') D1(14)/NY,D2(14)/NY
        WRITE(5,'(12H LOAD ENGY =,F11.0,4H GWh,
     &        18X,12H LOAD ENGY =,F11.0,4H GWh)') D1(2)/NY,D2(2)/NY
        WRITE(5,'(12H LOAD FACT =,F11.3,2H %,
     &        20X,12H LOAD FACT =,F11.3,2H %)') D1(3)/NY,D2(3)/NY
        WRITE(5,'(12H PEAK NETD =,F11.3,13H MW (DEMD-VR),
     &         9X,12H PEAK NETD =,F11.3,13H MW (DEMD-VR))')
     &         D1(4)/NY,D2(4)/NY
        WRITE(5,'(12H VR E USED =,F11.0,4H GWh,
     &        18X,12H VR E USED =,F11.0,4H GWh)') D1(5)/NY,D2(5)/NY
        WRITE(5,'(12H VR E LOST =,F11.0,4H GWh,
     &        18X,12H VR E LOST =,F11.0,4H GWh)') D1(6)/NY,D2(6)/NY
        WRITE(5,'(14H Indices for a,F6.2,6H% LFU:,
     &        19X,14H Indices for a,F6.2,6H% LFU:)') PLFU1,PLFU2
        WRITE(5,'(12H LOLE(AM)  =,F11.6,4H d/y,
     &        18X,12H LOLE(AM)  =,F11.6,4H d/y)') D1(7)/NY,D2(7)/NY
        WRITE(5,'(12H LOLE(PM)  =,F11.6,4H d/y,
     &        18X,12H LOLE(PM)  =,F11.6,4H d/y)') D1(8)/NY,D2(8)/NY
        WRITE(5,'(12H LOLE      =,F11.6,4H d/y,
     &        18X,12H LOLE      =,F11.6,4H d/y)') LOLEAM1,LOLEAM2
        WRITE(5,'(12H LOLEV **  =,F11.6,9H events/y,
     &        13X,12H LOLEV **  =,F11.6,9H events/y)') D1(9)/NY,D2(9)/NY
        WRITE(5,'(12H LOLEMW    =,F11.0,3H MW,
     &        19X,12H TIE LOLE  =,F11.6,4H d/y)') D1(11)/D1(10),
     &        LOLEAM2-LOLEAM1                                           ! the increase in area 2 LOLE due to the ties
        WRITE(5,'(12H LOLH      =,F11.6,4H h/y,
     &        18X,12H LOLH      =,F11.6,4H h/y)') D1(10)/NY,D2(10)/NY
        WRITE(5,'(12H LOLH/event=,F11.6,8H h/event,
     &        14X,12H LOLH/event=,F11.6,8H h/event)')
     &        D1(10)/D1(9),D2(10)/D2(9)
        WRITE(5,'(12H EUE MWh   =,F11.0,4H MWh,
     &        18X,12H TIE LINES =,F11.3,4H h/y)') D1(11)/NY,D2(11)/NY
        WRITE(5,'(12H EUE ppm   =,F11.2,7H pu ppm,
     &        15X,12H TIE LOLE  =,F11.1,2H %)') D1(11)/D1(12)*1.E6,
     &        LOLEAM2/LOLEAM1*100.-100.
        WRITE(5,'(12H EUE %     =,F11.6,2H %,
     &        20X,12H TIE LOLH  =,F11.1,2H %)') D1(11)/D1(12)*100.,
     &        D2(10)/D1(10)*100.-100.
        WRITE(5,'(6H LOLP>,1PE5.0E1,1H=,I11  ,7H hrs/yr,
     &        15X,6H LOLP>,1PE5.0E1,1H=,I11  ,7H hrs/yr)')
     &        SLOLP1,INT(D1(13)+.5)/NY,SLOLP2,INT(D2(13)+.5)/NY
      ENDIF
      IF(JS.EQ.0.AND.TLOLE2.GT.0.) THEN
        WRITE(*,'(/32HS   = step change logic for DSF2)')
        WRITE(*,'(26HDSF2 = demand scale factor)')
        WRITE(*,'(26HDDSF = per unit delta DSF2)')
        WRITE(*,'(32HS   LOLE d/y     DSF2       DDSF)')
        JS=1
      ENDIF
      IF(IP.EQ.1.AND.NVS2.GT.0) THEN
        WRITE(5,35) SLOLP2,(100.*VRES2(I)/D2(13)/SF2(I),I=1,NVS2)
  35    FORMAT(/' Variable Resource Capacity Credits Max Net Demand hrs'
     &    ,' for LOLP>',F8.6/20(F6.1,1H%))
      ENDIF
      IF(TLOLE2.GT.0.) WRITE(*,'(I1,F11.7,2X,G12.7,F6.3)')
     &  JS,LOLEAM2,DSF2,DDSF
      IF(IP.EQ.1) GOTO 20                                               ! skip iterative target LOLE or we are ready to wrap it up
      IF(ABS(TLOLE2-LOLEAM2).LE..0001*TLOLE2) THEN                      ! meets an excellent LOLE result
        IP=1                                                            ! make another pass so printout can happen
        GOTO 15
      ENDIF
      IF(ITER.EQ.1) THEN                                                ! first iteration we have to guess a delta step
        DSF0=DSF2                                                       ! remember where we came from on DSF
        LOLE0=LOLEAM2
        IF(TLOLE2.GE.LOLEAM2) THEN
          DSF2=DSF2*(1.+DDSF)                                           ! first iteration jump up 2% if too low
          JS=2
        ELSE
          DSF2=DSF2*(1.-DDSF)                                           ! first iteration jump down 2% if too high
          JS=3
        ENDIF                                                           ! notice we did not reduce the 2% step size because we may have to take another 2% step
        GOTO 15
      ENDIF
      IF(ABS(LOLEAM2-LOLE0).LE..00001*TLOLE2) THEN                      ! the change in LOLE was small on this iteration so wrap things up
        IP=1                                                            ! make another pass so the data can be printed out
        GOTO 15
      ENDIF
      IF(ABS(TLOLE2-LOLE0).LE..2*TLOLE2.OR.ABS(TLOLE2-LOLEAM2).LE.
     &  .2*TLOLE2) THEN
        D2(1) = (DSF2-DSF0)*(TLOLE2-LOLE0)/(LOLEAM2-LOLE0) + DSF0       ! use a Newton convergence when one end is close enough to the solution
        DSF0 = DSF2
        LOLE0= LOLEAM2
        DSF2  = D2(1)
          JS=4
        GOTO 15
      ENDIF
      IF(TLOLE2.LT.LOLEAM2.AND.TLOLE2.GT.LOLE0) THEN
        DDSF=DDSF/2.
        DSF2=DSF2*(1.-DDSF)
          JS=5
        GOTO 15
      ENDIF
      IF(TLOLE2.GT.LOLEAM2.AND.TLOLE2.LT.LOLE0) THEN
        DDSF=DDSF/2.
        DSF2=DSF2*(1.+DDSF)
          JS=6
        GOTO 15
      ENDIF
      DSF0 = DSF2
      LOLE0= LOLEAM2
      IF(TLOLE2.GT.LOLEAM2.AND.TLOLE2.GT.LOLE0) THEN
        DSF2=DSF2*(1.+DDSF)                                             ! load is too low increase the load
          JS=7
        GOTO 15
      ENDIF
      IF(TLOLE2.LT.LOLEAM2.AND.TLOLE2.LT.LOLE0) THEN
        DSF2=DSF2*(1.-DDSF)                                             ! load is too high decrease the load
          JS=8
        GOTO 15
      ENDIF
  15  CONTINUE
  20  WRITE(*,*)' '
      WRITE(5,36)
  36  FORMAT(/'** LOLE counts events per 24 hours and LOLEV in this prog
     &ram counts events per 12 hours.'/'   Sequential MC in different pr
     &ograms will give different LOLEV events counts making it'/
     &'   impossible to get an exact count.  LOLE, LOLH, EUE are exact v
     &alue points in 0 time.')
      CLOSE(5)
      OPEN(5,FILE='OP.TXT')                                             ! open text output report file so we can read the numbers and write them to OP.csv
      WRITE(6,'(/A199,1PE5.0E1,4H hrs)') '          YEAR,  PKDEMD,     %
     &RESV,LoadGWh,%LDFact,PKnetMW,    VRGWh,      VRlost,        %LFU,L
     &OLE(AM),LOLE(PM),LOLE(d/y),LOLEV(#/y),LOLET(d/y),LOLH(h/y),LOLH(h/
     &e),LOLHT(h/y),LOLET(%),LOLHT(%),LOLP>',SLOLP1
      READ(5,'(////////////46X,A4)') G2
  18  READ(5,'(/46X,A4/7(/57X,F11.0)/59X,F6.0,11(/57X,F11.0))',ERR=19)
     & G2,D2
      WRITE(6,'(A4,1H,F11.3,1H,F11.1,1H,F11.0,1H,F11.3,1H,F11.3,1H,F11.0
     &,1H,F11.0,1H,F11.2,1H,F11.6,1H,F11.6,1H,F11.6,1H,F11.6,1H,F11.6,1H
     &,F11.6,1H,F11.6,1H,F11.3,1H,F11.1,1H,F11.1,1H,F11.0,1H,)') G2,D2
      GOTO 18
  19  CLOSE(5)
      CLOSE(6)
      CLOSE(7)
      WRITE(*,*) 'SEE OP.TXT OP.CSV OPH.CSV FILES'
      WRITE(*,*) 'PRESS ENTER TO END THIS PROGRAM'
      READ(*,'(A1)') C222
      STOP
   9  WRITE(*,'(16H ERROR READING: ,A220)') C221
      WRITE(*,*) 'PRESS ENTER TO END THIS PROGRAM'
      READ(*,'(A1)') C221
      STOP
  26  WRITE(*,'(16H ERROR READING: ,A220)') C222
      WRITE(*,*) 'PRESS ENTER TO END THIS PROGRAM'
      READ(*,'(A1)') C222
      STOP
  16  WRITE(*,*) 'OPEN FILE ERROR: ',G1
      STOP
  24  WRITE(*,*) 'OPEN FILE ERROR: ',G2
      STOP
  17  WRITE(*,*) 'OPEN FILE ERROR: ',H1
      STOP
  25  WRITE(*,*) 'OPEN FILE ERROR: ',H2
      STOP
      END
